#-*- coding: utf-8 -*-

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# (C) 2012 - Pierre-Yves Chibon <pingou@pingoured.fr>

"""
Spec class, handles the read/write of the spec file
"""


import datetime
import os
import re
import sys
import textwrap
from jinja2 import Template
try:
    from pypi2spec import get_logger, get_rpm_tag, Pypi2specError
except ImportError:
    from __init__ import get_logger, get_rpm_tag, Pypi2specError


def format_description(description):
    """ Format the description as required by rpm """

    # Return the description unadulterated if it is already valid.
    if not any([len(line) > 75 for line in description.split('\n')]):
        return description

    # Otherwise, run it through the textwrap module.  This can have a
    # destructive affect on reStructuredText markup.
    wrapper = textwrap.TextWrapper(width=75)
    # First, remove trailing whitespace from every line
    cleaned = '\n'.join([l.strip() for l in description.split('\n')])
    # Format each paragraph into 75 chars per line and re-join
    return '\n\n'.join([wrapper.fill(p) for p in cleaned.split('\n\n')])


def format_dependencies(dependencies):
    """ Format the dependencies cleanning them as much as possible for rpm.
    """
    ignorelist = ['python']
    # Regular expression used to determine whether the string is a
    # version number
    versionmotif = re.compile('\d\.\d\.?\d?')
    char = {
            '\r': '',
            '(': ' ',
            ')': ' ',
            ',': ' ',
            '  ': ' ',
            }

    for key in char.keys():
        dependencies = dependencies.replace(key, char[key])
    dep_list = []

    for dep in dependencies.split(' '):
        if dep.strip():
            if  not ">" in dep \
            and not "<" in dep \
            and not "=" in dep \
            and len(versionmotif.findall(dep)) == 0 \
            and dep.strip() not in ignorelist:
                dep = dep.strip()
                dep_list.append(dep)

    return ' '.join(dep_list).strip()


class Spec:
    """
    Spec Class
        Write the spec file.
    """

    def __init__(self, settings, package=None):
        """ Constructor.
        """
        self.package = package
        self.settings = settings
        self.__dict = {}
        self.log = get_logger()
        self.spec = None

    def fill_spec_info(self):
        """ Fills the different variable required for the spec file. """
        self.log.info('Filling spec variable from info collected')
        self.__dict['name'] = self.package.name
        self.__dict['arch'] = self.package.arch
        self.__dict['version'] = self.package.version
        self.__dict['summary'] = self.package.summary
        self.__dict['license'] = self.package.license
        self.__dict['description'] = format_description(
            self.package.description)
        self.__dict['URL'] = self.package.url
        self.__dict['source0'] = self.package.source0
        self.__dict['packager'] = self.settings.get('packager')
        self.__dict['email'] = self.settings.get('email')
        self.__dict['date'] = datetime.datetime.now(
            ).strftime("%a %b %d %Y")

    def get_specfile(self):
        """ Return the path to the spec file.
        """
        specdir = get_rpm_tag('_specdir')
        specname = 'python-%s.spec' % self.package.name
        return '%s/%s' % (specdir, specname)

    def read_specfile(self):
        """ Read the specfile present in the spec directory.
        """
        specfile = self.get_specfile()
        if os.path.exists(specfile) and os.path.isfile(specfile):
            self.log.info('Reading file %s' % specfile)
            try:
                stream = open(specfile, 'r')
                self.spec = stream.read()
                stream.close()
            except IOError, err:
                self.log.info('Cannot read the file %s' % specfile)
                self.log.debug('ERROR: %s' % err)

    def get_template(self):
        """ Read the empty template and fills it with the information
        retrieved.
        """
        template = '%s/specfile.tpl' % os.path.dirname(__file__)
        self.log.info('Filling spec template')
        try:
            stream = open(template, 'r')
            tplfile = stream.read()
            stream.close()
            mytemplate = Template(tplfile)
            self.spec = mytemplate.render(self.__dict)
        except IOError, err:
            self.log.debug('ERROR: %s' % err)
            raise Pypi2specError('Cannot read the file %s' % template)

    def write_spec(self, verbose=False):
        """ Write down the spec to the spec directory as returned by rpm.
        """
        specfile = self.get_specfile()
        self.log.info('Writing file %s' % specfile)
        try:
            stream = open(specfile, 'w')
            stream.write(self.spec)
            stream.close()
            self.log.debug('Spec file writen: %s' % specfile)
            print 'Spec file writen: %s' % specfile
        except IOError, err:
            print 'Cannot write the file %s' % specfile
            self.log.debug('ERROR: %s' % err)



#!/usr/bin/env python
import sys
import re

from setuptools import setup
from pypi2spec import __version__

description = "Small library to help you generate spec file for pypi project."

long_description = """
pypi2spec makes you life easier at packaging pypi project for Fedora.
"""

download_url = "http://github.com/pypingou/pypi2spec-%s.tar.gz" % __version__

requirements = [
    'rdflib',
    'jinja2',
]

try:
    import argparse
except ImportError:
    requirements.append('argparse')

setup(
    name='pypi2spec',
    version=__version__,
    description=description,
    author="Pierre-Yves Chibon",
    author_email="pingou@pingoured.fr",
    maintainer="Pierre-Yves Chibon",
    maintainer_email="pingou@pingoured.fr",
    url="http://github.com/pypingou/pypi2spec",
    license="GPLv3+",
    long_description=long_description,
    download_url=download_url,
    packages=['pypi2spec'],
    include_package_data=True,
    install_requires=requirements,
    entry_points="""
    [console_scripts]
    pypi2spec = pypi2spec:main
    """,
    classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Environment :: Console",
        ],

)



#-*- coding: utf-8 -*-

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# (C) 2012 - Pierre-Yves Chibon <pingou@pingoured.fr>


import argparse
import ConfigParser
import logging
import shutil
import tarfile
import os
import urllib2

from subprocess import Popen, PIPE
from tarfile import TarError

logging.basicConfig()
LOG = logging.getLogger('Pypi2spec')
#LOG.setLevel('DEBUG')
__version__ = '0.2.0'


def create_conf(configfile):
    """Check if the provided configuration file exists, generate the
    folder if it does not and return True or False according to the
    initial check.

    :arg configfile, name of the configuration file looked for.
    """
    if not os.path.exists(configfile):
        dirn = os.path.dirname(configfile)
        if not os.path.exists(dirn):
            os.makedirs(dirn)
        return True
    return False


def save_config(configfile, parser):
    """"Save the configuration into the specified file.

    :arg configfile, name of the file in which to write the configuration
    :arg parser, ConfigParser object containing the configuration to
    write down.
    """
    conf = open(configfile, 'w')
    parser.write(conf)
    conf.close()


def get_logger():
    """ Return the logger. """
    return LOG


def get_rpm_tag(tag):
    """" Reads the .rpmmacros and set the values accordingly
    Code from José Matos.
    :arg tag, the rpm tag to find the value of
    """
    dirname = Popen(["rpm", "-E", '%' + tag], stdout=PIPE).stdout.read()[:-1]
    return dirname


def move_sources(fullpath, sources):
    """ Copy the tarball from its current location to the sourcedir as
    defined by rpm.

    :arg fullpath, the fullpath to the sources in their current location.
    :arg sources, the name of the file in which the origin will be copied.
    """
    sourcedir = get_rpm_tag('_sourcedir')
    dest = '%s/%s' % (sourcedir, sources)
    shutil.copyfile(fullpath, dest)


class Settings(object):
    """ Pypi2spec user config Setting"""
    # Editor to use in the spec
    packager = os.getlogin()
    # Editor email to use in the spec
    email = ''

    def __init__(self):
        """Constructor of the Settings object.
        This instanciate the Settings object and load into the _dict
        attributes the default configuration which each available option.
        """
        self._dict = {
                        'packager': self.packager,
                        'email': self.email,
                    }
        self.load_config('.config/pypi2spec', 'main')

    def load_config(self, configfile, sec):
        """Load the configuration in memory.

        :arg configfile, name of the configuration file loaded.
        :arg sec, section of the configuration retrieved.
        """
        parser = ConfigParser.ConfigParser()
        configfile = os.environ['HOME'] + "/" + configfile
        is_new = create_conf(configfile)
        parser.read(configfile)
        if not parser.has_section(sec):
            parser.add_section(sec)
        self.populate(parser, sec)
        if is_new:
            save_config(configfile, parser)

    def set(self, key, value):
        """ Set the value to the given key in the settings.

        :arg key, name of the parameter to set from the settings.
        :arg value, value of the parameter to set from the settings.
        """
        if not key in self._dict.keys():
            raise KeyError(key)
        self._dict[key] = value

    def get(self, key):
        """ Return the associated with the given key in the settings.

        :arg key, name of the parameter to retrieve from the settings.
        """
        if not key in self._dict.keys():
            raise KeyError(key)
        return self._dict[key]

    def populate(self, parser, section):
        """"Set option values from a INI file section.

        :arg parser: ConfigParser instance (or subclass)
        :arg section: INI file section to read use.
        """
        if parser.has_section(section):
            opts = set(parser.options(section))
        else:
            opts = set()

        for name in self._dict.iterkeys():
            value = None
            if name in opts:
                value = parser.get(section, name)
                parser.set(section, name, value)
                self._dict[name] = value
            else:
                parser.set(section, name, self._dict[name])


class Pypi2specError(Exception):
    """ Pypi2specError class
    Template for all the error of the project
    """

    def __init__(self, value):
        """ Constructor. """
        self.value = value

    def __str__(self):
        """ Represent the error. """
        return str(self.value)


class Pypi2spec(object):
    """ Pypi2spec main class whose goal is to get the all the info
    needed for the spec file.
    """

    def __init__(self, name):
        """ Constructor.
        :arg name, the name of the library on the pypi website.
        """
        self.name = name
        self.log = get_logger()
        self.version = ''
        self.summary = ''
        self.license = ''
        self.url = 'http://pypi.python.org/pypi/%s' % name
        self.source0 = ''
        self.source = ''
        self.arch = False

    def determine_arch(self):
        """ Determine if the package is arch or noarch by looking at the
        sources.
        Set arch to True if the package is arch dependant.
        Set arch to False if the package is noarch.
        Let arch to None if could not determine.
        """
        self.log.info('Determining if the package is arch dependant or not')
        extensions = ['c', 'C', 'cp', 'cpp', 'h', 'H', ]
        if os.path.exists(self.name):
            for root, dirs, files in os.walk(self.name):
                for entry in files:
                    if '.' in entry:
                        extension = entry.rsplit('.', 1)[1]
                        if extension in extensions \
                                or 'f' in extension \
                                or 'F' in extension:
                            self.arch = True
                            self.log.info('Package is arch dependant')
                            return
            self.arch = False
            self.log.info('Package is not arch dependant')
            return
        else:
            self.log.info(
                'Could not find the extracted source to search the arch')

    def download(self, force=False):
        """ Download the source of the package into the source directory
        which we retrieve from rpm directly.

        arg force, boolean whether to force the download of the sources
        even if they are on the system already.
        """
        sourcedir = get_rpm_tag('_sourcedir')
        sources = '%s/%s' % (sourcedir, self.source)

        if not force and os.path.exists(sources) and os.path.isfile(sources):
            self.log.info(
                "Sources are already present, no need to re-download")
            return

        url = self.source0.rsplit('/', 1)[0]
        url = '{url}/{source}'.format(url=url, source=self.source)
        self.log.info('Downloading %s' % url)

        remotefile = urllib2.urlopen(url)
        localfile = open(sources, 'w')
        localfile.write(remotefile.read())
        localfile.close()

    def extract_sources(self):
        """ Extract the sources into the current directory. """
        sourcedir = get_rpm_tag('_sourcedir')
        tarball = "%s/%s" % (sourcedir, self.source)
        self.log.info("Opening: %s" % tarball)
        try:
            tar = tarfile.open(tarball)
            tar.extractall()
            tar.close()
        except TarError, err:
            self.log.debug("Error while extracting the tarball")
            self.log.debug("ERROR: %s" % err)

    def remove_sources(self):
        """ Remove the source we extracted in the current working
        directory.
        """
        source = '%s-%s' % (self.name, self.version)
        self.log.info('Removing extracted sources: "%s"' % source)
        try:
            shutil.rmtree(source)
        except (IOError, OSError), err:
            self.log.info('Could not remove the extracted sources: "%s"'\
                % source)
            self.log.debug('ERROR: %s' % err)

    def retrieve_info(self):
        """ Retrieve all the information from pypi to fill up the spec
        file.
        """
        import rdflib
        g = rdflib.Graph()
        try:
            g.parse('http://pypi.python.org/pypi?:action=doap&name=%s' % \
                self.name, format='xml')
        except urllib2.HTTPError, err:
            self.log.debug('ERROR while downloading the doap file:\n  %s'
                % err)
            raise Pypi2specError('Could not retrieve information for the'
                'project "%s". Did you make a typo?' % self.name)
        DOAP = rdflib.Namespace('http://usefulinc.com/ns/doap#')
        RDFS = rdflib.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
        version_node = g.value(predicate=RDFS['type'],
            object=DOAP['Version'])
        self.version = g.value(subject=version_node,
            predicate=DOAP['revision'])
        project_node = g.value(predicate=RDFS['type'],
            object=DOAP['Project'])
        self.summary = g.value(subject=project_node,
            predicate=DOAP['shortdesc'])
        self.description = g.value(subject=project_node,
            predicate=DOAP['description'])
        self.source0 = g.value(subject=project_node,
            predicate=DOAP['download-page'])

        if not self.source0:
            pypi_base = 'http://pypi.python.org/packages/source/'
            self.source0 = pypi_base + '%s/%s/%s-%s.tar.gz' % \
                (self.name[0], self.name, self.name, self.version)

        self.source = '%s-%s.tar.gz' % (self.name, self.version)


class Pypi2specUI(object):
    """ Class handling the user interface. """

    def setup_parser(self):
        """ Command line parser. """
        self.parser = argparse.ArgumentParser(usage='%(prog)s [options]',
                prog='pypi2spec')
        self.parser.add_argument('--version', action='version',
            version='%(prog)s ' + __version__)
        self.parser.add_argument('package',
            help='Name of the pypi library to package.')
        self.parser.add_argument('--verbose', action='store_true',
            help='Give more info about what is going on.')
        self.parser.add_argument('--debug', action='store_true',
            help='Output bunches of debugging info.')

    def main(self):
        """ Main function.
        Entry point of the program.
        """
        try:
            from spec import Spec
            self.setup_parser()
            args = self.parser.parse_args()

            self.log = get_logger()
            if args.verbose:
                self.log.setLevel('INFO')
            if args.debug:
                self.log.setLevel('DEBUG')

            pypi = Pypi2spec(args.package)
            pypi.retrieve_info()
            pypi.download()
            pypi.extract_sources()
            pypi.determine_arch()
            pypi.remove_sources()
            settings = Settings()
            spec = Spec(settings, pypi)
            spec.fill_spec_info()
            spec.get_template()
            spec.write_spec()
        except Pypi2specError, err:
            print err


def main():
    return Pypi2specUI().main()

if __name__ == '__main__':
    main()
